import 'package:fff_info/theme.dart';
import 'package:flutter/material.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';
import 'package:intl/intl.dart';
import 'package:share/share.dart';

import 'api.dart';
import 'drawer.dart';

class StrikeListPage extends StatefulWidget {
  StrikeListPage({
    Key key,
  }) : super(key: key);

  @override
  _StrikeListPageState createState() => _StrikeListPageState();
}

class _StrikeListPageState extends State<StrikeListPage> {
  List _strikeList = List();
  IconData _appBarIcon = Icons.search;
  TextEditingController _searchController = TextEditingController();
  Set<String> _countries = Set.from(['All']);
  String _selectedCountry = 'All';

  String _query = "";
  DateTime _before;
  DateTime _after = DateTime.now();

  Set _eventTypes = Set();
  bool _onlyShowComingEvents = true;
  bool _onlyShowEventsBefore = false;

  @override
  Widget build(BuildContext context) {
    // This method is rerun every time setState is called, for instance as done
    // by the _incrementCounter method above.
    //
    // The Flutter framework has been optimized to make rerunning build methods
    // fast, so that you can just rebuild anything that needs updating rather
    // than having to individually change instances of widgets.
    if (_strikeList.isEmpty) {
      Api().eventData().then((data) {
        data.sort(
            (a, b) => parseDate(a['date']).compareTo(parseDate(b['date'])));
        data.forEach((elem) => _countries.add(elem['country']));
        _countries.removeWhere((value){return value==null||value=='';});
        setState(() {
          _strikeList = data;
        });
      });
    }
    List filteredItems = List.from(_strikeList);
    if (_query != "")
      filteredItems = _searchStrikes(filteredItems, query: _query);
    if (_onlyShowEventsBefore)
      filteredItems = _filterByDate(filteredItems, before: _before);
    if (_onlyShowComingEvents)
      filteredItems = _filterByDate(filteredItems, after: _after);
    if(_selectedCountry!='All') filteredItems.removeWhere((elem){return(elem['country']!=_selectedCountry);});
    filteredItems = _filterByRecurrance(filteredItems);
    return Scaffold(
      drawer: ClimateDrawer(),
      appBar: AppBar(
          // Here we take the value from the MyHomePage object that was created by
          // the App.build method, and use it to set our appbar title.
          title: (_appBarIcon == Icons.search)
              ? Text("Strike List")
              : Semantics(
                  label: 'Search strikes by keywords',
                  child: TextField(
                    autofocus: true,
                    controller: _searchController,
                    onChanged: (newVal) {
                      setState(() {
                        _query = newVal;
                      });
                    },
                    //decoration: InputDecoration(labelText: 'Search strikes',),
                  ),
                ),
          actions: <Widget>[
            IconButton(
              tooltip: 'Toggle search field',
              icon: Icon(_appBarIcon),
              onPressed: () {
                if (_appBarIcon == Icons.search) {
                  setState(() {
                    _appBarIcon = Icons.close;
                  });
                } else {
                  setState(() {
                    _appBarIcon = Icons.search;
                    _query = '';
                  });
                }
              },
            )
          ]),
      body: Column(
        children: <Widget>[
          Text(
            "Strike List",
            style: Theme.of(context).textTheme.display1,
          ),
          (_strikeList.isNotEmpty)
              ? (filteredItems.isNotEmpty)
                  ? Expanded(
                      child: Column(
                      children: <Widget>[
                        Text("Matching events: " +
                            filteredItems.length.toString() +
                            " of " +
                            _strikeList.length.toString() +
                            ' (' +
                            ((filteredItems.length / _strikeList.length) * 100)
                                .round()
                                .toString() +
                            '%)'),
                        Expanded(
                          child: ListView.builder(
                            itemCount: filteredItems.length,
                            itemBuilder: (BuildContext context, int index) {
                              return (StrikeDetailTile(
                                strikeData: filteredItems[index],
                              ));
                            },
                          ),
                        ),
                      ],
                    ))
                  : Center(
                      child: Padding(
                      padding: const EdgeInsets.all(8.0),
                      child: Text('No matching events found.'),
                    ))
              : Center(
                  child: Padding(
                    padding: const EdgeInsets.all(8.0),
                    child: CircularProgressIndicator(),
                  ),
                ),
          Text(
            "Source: https://www.fridaysforfuture.org/events/map",
            style: Theme.of(context).textTheme.caption,
          )
        ],
      ),
      floatingActionButton: FloatingActionButton.extended(
        onPressed: _toggleFilterMenu,
        label: Text('Filter'),
        icon: Icon(FontAwesomeIcons.filter),
      ),
      floatingActionButtonLocation: FloatingActionButtonLocation.centerFloat,
    );
  }

  List _filterByDate(List events, {DateTime after, DateTime before}) {
    if (before != null) {
      events.removeWhere((item) {
        return (parseDate(item['date']).isAfter(before));
      });
    }
    if (after != null) {
      events.removeWhere((item) {
        bool isBefore = parseDate(item['date']).isBefore(after);
        return (isBefore);
      });
    }
    return (events);
  }

  List _searchStrikes(List events, {String query}) {
    events.removeWhere((strikeItem) {
      parseDate(strikeItem['date']);
      String strikeText = strikeItem['country'] +
          strikeItem['town'] +
          strikeItem['location'] +
          strikeItem['event_type'] +
          strikeItem['contact_name'] +
          strikeItem['notes'] +
          strikeItem['org'];
      return (!strikeText.toLowerCase().contains(query.toLowerCase()));
    });
    return (events);
  }

  Future<Null> _selectBeforeDate(BuildContext context) async {
    final DateTime picked = await showDatePicker(
        context: context,
        initialDate: (_before != null) ? _before : DateTime.now(),
        firstDate: DateTime(2015, 8),
        lastDate: DateTime(2101));
    if (picked != null && picked != _before)
      setState(() {
        _before = picked;
      });
  }

  void _toggleFilterMenu() {
    showModalBottomSheet(
        context: context,
        builder: (context) {

          return (Container(
            child: ListView(
              children: <Widget>[
                ListTile(
                  title: Row(
                    children: <Widget>[
                      Text('Country: '),
                      DropdownButton(
                        value: _selectedCountry,
                        icon: Icon(FontAwesomeIcons.globeAsia),
                        iconSize: 24,
                        elevation: 16,
                        style: TextStyle(
                            color: ClimateTheme.of(context).primaryColor),
                        underline: Container(
                          height: 2,
                          color: ClimateTheme.of(context).accentColor,
                        ),
                        onChanged: (String newValue) {
                          setState(() {
                            _selectedCountry = newValue;
                          });
                        },
                        items: _countries.map<DropdownMenuItem<String>>((String value) {
                          return DropdownMenuItem<String>(
                            value: value,
                            child: Text(value),
                          );
                        }).toList(),
                      )
                    ],
                  ),
                ),
                SwitchListTile(
                  title: Text('Only comming'),
                  value: _onlyShowComingEvents,
                  onChanged: (newValue) {
                    if (newValue) {
                      setState(() {
                        _onlyShowComingEvents = true;
                        _after = DateTime.now();
                      });
                    } else {
                      setState(() {
                        _onlyShowComingEvents = false;
                        _after = null;
                      });
                    }
                    Navigator.pop(context);
                  },
                ),
                SwitchListTile(
                  title: Row(
                    children: <Widget>[
                      Text('Only before: '),
                      RaisedButton(
                        child: Text((_before == null)
                            ? 'Not selected'
                            : "${_before.toLocal()}"),
                        onPressed: () {
                          _selectBeforeDate(context);
                        },
                      ),
                    ],
                  ),
                  value: _onlyShowEventsBefore,
                  onChanged: (newValue) {
                    if (newValue) {
                      setState(() {
                        _onlyShowEventsBefore = true;
                      });
                    } else {
                      setState(() {
                        _onlyShowEventsBefore = false;
                        _after = null;
                      });
                    }
                    Navigator.pop(context);
                  },
                ),
                ListTile(
                  leading: Icon(FontAwesomeIcons.retweet),
                  title: Text('Recurrence:'),
                ),
                CheckboxListTile(
                  title: Text('Every Friday'),
                  value: _eventTypes.contains('Every Friday'),
                  onChanged: (selected) {
                    setState(() {
                      (selected)
                          ? _eventTypes.add('Every Friday')
                          : _eventTypes.remove('Every Friday');
                    });
                    Navigator.pop(context);
                  },
                ),
                CheckboxListTile(
                  title: Text('Weekly'),
                  value: _eventTypes.contains('Weekly'),
                  onChanged: (selected) {
                    setState(() {
                      (selected)
                          ? _eventTypes.add('Weekly')
                          : _eventTypes.remove('Weekly');
                    });
                    Navigator.pop(context);
                  },
                ),
                CheckboxListTile(
                  title: Text('Once only'),
                  value: _eventTypes.contains('Once only'),
                  onChanged: (selected) {
                    setState(() {
                      (selected)
                          ? _eventTypes.add('Once only')
                          : _eventTypes.remove('Once only');
                    });
                    Navigator.pop(context);
                  },
                ),
                CheckboxListTile(
                  title: Text(
                    'Others',
                    style: TextStyle(fontStyle: FontStyle.italic),
                  ),
                  value: _eventTypes.isEmpty,
                  onChanged: (selected) {
                    setState(() {
                      if (selected) _eventTypes.clear();
                    });
                    Navigator.pop(context);
                  },
                ),
              ],
            ),
          ));
        });
  }

  List _filterByRecurrance(List filteredItems) {
    if (_eventTypes.isNotEmpty)
      filteredItems.removeWhere((item) {
        return (!_eventTypes.contains(item['recurrence']));
      });
    return filteredItems;
  }
}

class StrikeDetailTile extends StatelessWidget {
  Map strikeData;

  StrikeDetailTile({@required this.strikeData});

  @override
  Widget build(BuildContext context) {
    // TODO: implement build
    return ExpansionTile(
      title: Text(
        strikeData['country'] + ', ' + strikeData['town'],
        style: Theme.of(context)
            .textTheme
            .title
            .copyWith(fontWeight: FontWeight.w500),
      ),
      children: <Widget>[
        Padding(
          padding: EdgeInsets.all(8.0),
          child: StrikeDetail(
            strikeData: strikeData,
            hideHeader: true,
          ),
        )
      ],
    );
  }
}

class StrikeDetail extends StatelessWidget {
  Map strikeData;
  bool hideHeader;

  StrikeDetail({@required this.strikeData, this.hideHeader = false});

  @override
  Widget build(BuildContext context) {
    return Card(
        child: Padding(
      padding: const EdgeInsets.all(16.0),
      child: ListView(
        shrinkWrap: true,
        physics: NeverScrollableScrollPhysics(),
        children: <Widget>[
          //(!hideHeader) ? Text(strikeData['country']+', '+strikeData['city'], style: Theme.of(context).textTheme.headline,) : null,
          ListTile(
            leading: Icon(FontAwesomeIcons.calendarCheck),
            title: Text(DateFormat("EEEE, dd.MM.yyyy")
                    .format(parseDate(strikeData['date'])) +
                ', ' +
                strikeData['time']),
          ),
          ListTile(
            leading: Icon(FontAwesomeIcons.mapMarkedAlt),
            title: Text(strikeData['location']),
          ),
          ListTile(
            leading: Icon(FontAwesomeIcons.retweet),
            title: Text('Recurrance: ' + strikeData['recurrence']),
          ),
          ListTile(
            leading: Icon(FontAwesomeIcons.hammer),
            title: Text('Event type: ' + strikeData['event_type']),
          ),
          ListTile(
            leading: Icon(Icons.share),
            title: Text('Share this event'),
            onTap: () {
              Share.share('Climate strike on ' +
                  DateFormat("EEEE, dd.MM.yyyy")
                      .format(parseDate(strikeData['date'])) +
                  ', ' +
                  strikeData['time'] +
                  '\n\nin ' +
                  strikeData['country'] +
                  ', ' +
                  strikeData['town'] +
                  '\n' +
                  'Event type: ' +
                  strikeData['event_type'] +
                  '\n' +
                  'Recurrance: ' +
                  strikeData['recurrence']);
            },
          ),
        ],
      ),
    ));
  }
}

DateTime parseDate(String date) {
  Map monthNames = {
    'Jan': 01,
    'Feb': 02,
    'Mar': 03,
    'Apr': 04,
    'May': 05,
    'Jun': 06,
    'Jul': 07,
    'Aug': 08,
    'Sep': 09,
    'Oct': 10,
    'Nov': 11,
    'Dec': 12,
  };
  int day = int.parse(date.substring(5, 7));
  int month = monthNames[date.substring(8, 11)];
  int year = int.parse(date.substring(12, 16));

  return DateTime(year, month, day);
}

/*
class SingleMarkerMap extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    // TODO: implement build
    return new FlutterMap(
      options: new MapOptions(
        center: new LatLng(51.5, -0.09),
        zoom: 13.0,
      ),
      layers: [
        TileLayerOptions(
            urlTemplate:
            'https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png',
            subdomains: ['a', 'b', 'c']),
        MarkerLayerOptions(
          markers: [
            new Marker(
              width: 80.0,
              height: 80.0,
              point: LatLng(51.5, -0.09),
              builder: (ctx) => new Container(
                child: new FlutterLogo(),
              ),
            ),
          ],
        ),
      ],
    );
  }
}*/
